/*
 * sb8255.c
 *
 * 8255 interface for Python
 *
 * Author  : Hye-Shik Chang <perky@FreeBSD.org>
 * Date    : $Date: 2002/07/08 14:54:54 $
 * Created : 10 July 2002
 *
 * $Revision: 1.5 $
 */

static char *version =
"$Revision: 1.5 $";

#include "Python.h"
#include "8255.h"

static PyObject *ErrorObject;

static char sb8255_outp__doc__[] = "outp(port, value): Output to port a value.";

static PyObject *
sb8255_outp(PyObject *self, PyObject *args)
{
    int value, port;

    if (!PyArg_ParseTuple(args, "ii:outp", &port, &value))
        return NULL;

    outport(port, value);

    Py_INCREF(Py_None);
    return Py_None;
}

static char sb8255_inp__doc__[] = "inp(port): Input from port a value.";

static PyObject *
sb8255_inp(PyObject *self, PyObject *args)
{
    int port;

    if (!PyArg_ParseTuple(args, "i:inp", &port))
        return NULL;

    return PyInt_FromLong((long)inport(port));
}

static char sb8255_clear__doc__[] = "clear(port): Clear the 8255 port.";

static PyObject *
sb8255_clear(PyObject *self, PyObject *args)
{
    int port;

    if (!PyArg_ParseTuple(args, "i:clear", &port))
        return NULL;

    clear_port(port);

    Py_INCREF(Py_None);
    return Py_None;
}

static char sb8255_reset__doc__[] = "reset(): Reset all 8255 ports.";

static PyObject *
sb8255_reset(PyObject *self, PyObject *args)
{
    if (!PyArg_ParseTuple(args, ":reset"))
        return NULL;

    mask_8255();

    Py_INCREF(Py_None);
    return Py_None;
}

static char sb8255_init__doc__[] = \
"init(mode_a=0, mode_b=0, mode_c_upper=0, mode_c_lower=0): Initialize 8255.\n\
(0 for output, 1 for input)";

static PyObject *
sb8255_init(PyObject *self, PyObject *args)
{
    int m0 = 0, m1 = 0, m2 = 0, m3 = 0;

    if (!PyArg_ParseTuple(args, "|iiii:init", &m0, &m1, &m2, &m3))
        return NULL;

/*
 |  1000 0000   output:0 input:1
 |     | | |+-- Port C lower
 |     | | +--- Port B
 |     | +----- Port C upper
 |     +------- Port A
 */

#define truemask(cond, value) ((cond) ? (value) : 0)
    init_8255( 0x80 | 
               truemask(m0, 0x10) |
               truemask(m1, 0x02) |
               truemask(m2, 0x08) |
               truemask(m3, 0x01) );
#undef truemask

    Py_INCREF(Py_None);
    return Py_None;
}


/* List of methods defined in the module */

#define meth(name, func, doc) {name, (PyCFunction)func, METH_VARARGS, doc}
#define meth_kw(name, func, doc) {name, (PyCFunction)func, METH_VARARGS|METH_KEYWORDS, doc}

static struct PyMethodDef sb8255_methods[] = {
  meth("outp",  sb8255_outp,    sb8255_outp__doc__),
  meth("inp",   sb8255_inp,     sb8255_inp__doc__),
  meth("reset", sb8255_reset,   sb8255_reset__doc__),
  meth("clear", sb8255_clear,   sb8255_clear__doc__),
  meth("init",  sb8255_init,    sb8255_init__doc__),
  {NULL, NULL},
};


#define SET_INTCONSTANT(dict, value) \
            PyDict_SetItemString(dict, #value, PyInt_FromLong((long) value))
#define SET_STRCONSTANT(dict, value) \
            PyDict_SetItemString(dict, #value, PyString_FromString(value))
#define SET_CHARCONSTANT(dict, value) \
            PyDict_SetItemString(dict, #value, PyString_FromFormat("%c", value))

/* Initialization function for the module */

void
initsb8255(void)
{
    PyObject *m, *d;

    /* Create the module and add the functions */
    m = Py_InitModule("sb8255", sb8255_methods);

    /* Add some symbolic constants to the module */
    d = PyModule_GetDict(m);

    SET_INTCONSTANT(d, PORT_A);
    SET_INTCONSTANT(d, PORT_B);
    SET_INTCONSTANT(d, PORT_C);
    SET_INTCONSTANT(d, CTRL_REG);

    PyDict_SetItemString(d, "__version__", PyString_FromString(version));

    ErrorObject = PyErr_NewException("sb8255.error", NULL, NULL);
    PyDict_SetItemString(d, "error", ErrorObject);
    Py_DECREF(ErrorObject);

    /* Check for errors */
    if (PyErr_Occurred())
        Py_FatalError("can't initialize the sb8255 module");
}

/*
 * $Id: sb8255.c,v 1.5 2002/07/08 14:54:54 perky Exp $
 * ex: ts=8 sts=4 et
 */
